#!/bin/bash
set -e

move_asterisk_to_oldstable() {
    local lts_codename="$1"
    local destination="xivo-${lts_codename}-oldstable"
    local source="xivo-${lts_codename}"
    local prefix="debian"
    # If we were given 2023.05 as lts_codename then
    # - we must append -latest to the source
    # - and change the prefix to archive
    if [[ "$lts_codename" =~ [0-9]{4}\.[0-9]{2} ]]; then 
        source="xivo-${lts_codename}-latest"
        prefix="archive"
    fi
    local package_name='asterisk'

    local repo_suffix=""
    [[ "$lts_codename" == "kuma" || "$lts_codename" == "2023.05" ]] && repo_suffix="-kuma"

    local package_version_raw
    package_version_raw=$(get_package_version_raw "$source" "$package_name")

    local snapshot_name_from_pkg_version
    snapshot_name_from_pkg_version=$(create_snapshot_name_from_pkg_version "$package_name" "$package_version_raw" "$repo_suffix")

    # Check if asterisk snapshot exists and handle accordingly
    if check_snapshot_exists "$snapshot_name_from_pkg_version"; then
        validate_snapshot_version "$snapshot_name_from_pkg_version" "$package_version_raw"
    else
        aptly snapshot filter "$source" "$snapshot_name_from_pkg_version" \
            "Name (~ asterisk*), Version (= ${package_version_raw})"
    fi

    # if lts exist
    if  check_snapshot_exists "$source"; then
        echo "$source exist"
        # if oldstable exist
        if check_snapshot_exists "$destination"; then
            echo "${destination} exist: renaming it to ${destination}-before-${snapshot_name_from_pkg_version}"
            aptly snapshot rename "${destination}" "${destination}-before-${snapshot_name_from_pkg_version}"

            echo "Updating ${destination} with asterisk ${package_version_raw}"
            aptly snapshot merge -no-remove "$destination" "${destination}-before-${snapshot_name_from_pkg_version}" "${snapshot_name_from_pkg_version}"
        else
            # Create new oldstable from lts
            echo "Creating ${destination} with asterisk ${package_version_raw}"
            aptly snapshot merge -no-remove "$destination" "${snapshot_name_from_pkg_version}"
        fi
    else
        echo "The lts ${source} was not found. It should not happen. Exiting."
        exit 1
    fi

    publish_if_needed "$destination" "$prefix" "yes" "no"
}

update_lts_from_candidate() {
    local lts_codename="$1"
    local destination="xivo-${lts_codename}"
    local prefix="debian"
    # If we were given 2023.05 as lts_codename then
    # - we must append -latest to the destination
    # - and change the prefix to archive
    if [[ "$lts_codename" =~ [0-9]{4}\.[0-9]{2} ]]; then 
        destination="xivo-${lts_codename}-latest"
        prefix="archive"
    fi
    local source="xivo-${lts_codename}-candidate"
    local package_name="asterisk"

    local repo_suffix=""
    [[ "$lts_codename" == "kuma" || "$lts_codename" == "2023.05" ]] && repo_suffix="-kuma"

    # Check if source exists before merging
    if check_snapshot_exists "$source"; then
        echo "${source} exist"

        local package_version_raw
        package_version_raw=$(get_package_version_raw "$source" "$package_name")

        local snapshot_name_from_pkg_version
        snapshot_name_from_pkg_version=$(create_snapshot_name_from_pkg_version "$package_name" "$package_version_raw" "$repo_suffix")

        echo "Pull content from $source into $destination. with name: $destination"
        if check_snapshot_exists "$destination"; then
            echo "${destination} exist: renaming it to ${destination}-before-${snapshot_name_from_pkg_version}"
            aptly snapshot rename "${destination}" "${destination}-before-${snapshot_name_from_pkg_version}"

            echo "Updating ${destination} with asterisk ${package_version_raw}"
            aptly snapshot merge "$destination" "${destination}-before-${snapshot_name_from_pkg_version}" "$source"
        else
            echo "The lts ${destination} was not found. It should not happen. Exiting."
            exit 1
        fi
    else
        echo "Source snapshot $source does not exist. Skipping merge."
    fi

    publish_if_needed "$destination" "$prefix" "no" "no"
}


update_candidate_from_snapshot() {
    local lts_codename="$1"
    local source_snapshot="$2"
    local package_name="asterisk"
    local destination="xivo-${lts_codename}-candidate"
    local prefix="debian"
    # If we were given 2023.05 as lts_codename then
    # - we must change the prefix to archive
    if [[ "$lts_codename" =~ [0-9]{4}\.[0-9]{2} ]]; then 
        prefix="archive"
    fi

    local repo_suffix=""
    [[ "$lts_codename" == "kuma" || "$lts_codename" == "2023.05" ]] && repo_suffix="-kuma"

    local package_version_raw
    package_version_raw=$(get_package_version_raw "$source_snapshot" "$package_name")

    local snapshot_name_from_pkg_version
    snapshot_name_from_pkg_version=$(create_snapshot_name_from_pkg_version "$package_name" "$package_version_raw" "$repo_suffix")

    if check_snapshot_exists "$destination"; then
        echo "${destination} exist: renaming it to ${destination}-before-${snapshot_name_from_pkg_version}"
        aptly snapshot rename "${destination}" "${destination}-before-${snapshot_name_from_pkg_version}"

        echo "Updating ${destination} with asterisk ${package_version_raw}"
        aptly snapshot filter "$source_snapshot" "$destination" "Name (~ asterisk*), Version (= ${package_version_raw})"
    else
        echo "Creating ${destination} with asterisk ${package_version_raw}"
        aptly snapshot filter "$source_snapshot" "$destination" "Name (~ asterisk*), Version (= ${package_version_raw})"
    fi

    publish_if_needed "$destination" "$prefix" "yes" "yes"
}

publish_if_needed() {
    local snapshot="$1"
    local prefix="$2"
    local notautomatic="$3"
    local butautomaticupgrades="$4"

    if aptly publish show "$snapshot" "$prefix" > /dev/null 2>&1; then
        echo "Publication $prefix/$snapshot already exists. Switching it to new snapshot."
        aptly publish switch "${snapshot}" "${prefix}" "${snapshot}"
    else
        echo "Publication $prefix/$snapshot does not exist. Creating it."
        aptly publish snapshot -distribution="$snapshot" -notautomatic="$notautomatic" -butautomaticupgrades="$butautomaticupgrades" -architectures='amd64,source' "$snapshot" "$prefix"
    fi
}

check_snapshot_exists() {
    local snapshot="$1"

    if aptly snapshot show "$snapshot" > /dev/null 2>&1; then
        return 0
    else
        return 1
    fi
}

# Return all the (unique) versions of a package in a snapshot
get_package_versions_raw() {
    local snapshot="${1}"
    local package_name="${2}"

    local res
    res=$(aptly snapshot search "$snapshot" "$package_name" | \
                        grep -i "${package_name}_" | \
                        cut -d '_' -f2 | \
                        uniq)
    echo "${res}"
}
# Return the first version of a package in a snapshot
# by using the output of get_package_versions_raw
get_package_version_raw() {
    local snapshot="${1}"
    local package_name="${2}"

    local all_pkg_versions
    IFS=" " read -r -a all_pkg_versions <<< "$(get_package_versions_raw "$snapshot" "$package_name")"

    echo "${all_pkg_versions[0]}"
}

create_snapshot_name_from_pkg_version() {
    local package_name="${1}"
    local package_version_raw="${2}"
    local repo_suffix="${3}"

    echo "${package_name}-$(echo "${package_version_raw}" | cut -d '+' -f1,2 | sed -r 's/[:~\.+_]/-/g')${repo_suffix}"
}

validate_snapshot_version() {
    local snapshot_name="$1"
    local expected_version="$2"

    if ! aptly snapshot search "$snapshot_name" "Name (~ asterisk*), Version (= $expected_version)" > /dev/null 2>&1; then
        echo "ERROR: Snapshot corresponding to our asterisk version found: $snapshot_name"
        echo "ERROR: it does not contain the correct asterisk version"
        echo "ERROR: we were expecting version $expected_version"
        echo "This is an error. It should not happen."
        exit 1
    else
        echo "Snapshot $snapshot_name with version $expected_version found; will use it for merge."
    fi
}


pseudo_dry_run() {
    local lts_codename="$1"
    local source_snapshot="$2"

    local package_name="asterisk"
    # If we were given 2023.05 as lts_codename then
    # - we must append -latest to the destination
    # - and change the prefix to archive
    if [[ "$lts_codename" =~ [0-9]{4}\.[0-9]{2} ]]; then 
        lts="xivo-${lts_codename}-latest"
    else
        lts="xivo-${lts_codename}"
    fi
    local lts_candidate="xivo-${lts_codename}-candidate"
    local lts_oldstable="xivo-${lts_codename}-oldstable"

    local action_create_candidate
    local action_create_lts
    local action_create_oldstable
    local package_version_in_candidate_current
    local package_version_in_candidate_new
    local package_version_in_lts_current
    local package_version_in_lts_new
    local package_version_in_oldstable_current
    local package_version_in_oldstable_new

    if check_snapshot_exists "$lts_candidate"; then
        action_create_candidate="Already exists"
        package_version_in_candidate_current=$(get_package_version_raw "$lts_candidate" "$package_name")
    else
        action_create_candidate="Will be created"
        package_version_in_candidate_current="none"
    fi
    package_version_in_candidate_new=$(get_package_version_raw "$source_snapshot" "$package_name")

    if check_snapshot_exists "$lts"; then
        action_create_lts="Already exists"
        package_version_in_lts_current=$(get_package_version_raw "$lts" "$package_name")
    else
        action_create_lts="Will be created"
        package_version_in_lts_current="none"
    fi
    package_version_in_lts_new=$package_version_in_candidate_current

    if check_snapshot_exists "$lts_oldstable"; then
        action_create_oldstable="Already exists"
        package_version_in_oldstable_current=$(get_package_versions_raw "$lts_oldstable" "$package_name")
    else
        action_create_oldstable="Will be created"
        package_version_in_oldstable_current="none"
    fi
    package_version_in_oldstable_new="$package_version_in_lts_current $package_version_in_oldstable_current"

    echo ""
    echo "Here's what will happen:"
    echo "$lts_candidate :"
    echo "  - $action_create_candidate"
    echo "  - Current asterisk version: $package_version_in_candidate_current"
    echo "  - New asterisk version:     $package_version_in_candidate_new"
    echo ""
    echo "$lts :"
    echo "  - $action_create_lts"
    echo "  - Current asterisk version(s): $package_version_in_lts_current"
    echo "  - New asterisk version(s):     $package_version_in_lts_new"
    echo ""
    echo "$lts_oldstable :"
    echo "  - $action_create_oldstable"
    echo "  - Current asterisk version(s): $package_version_in_oldstable_current"
    echo "  - New asterisk version(s):     $package_version_in_oldstable_new"
    echo ""
}